#include <unistd.h>
#include <fcntl.h>
#include <signal.h>

#include <sys/time.h>

#include "fly.h"

#define RED    0
#define GREEN  1
#define BLUE   2

#define AVAILABLE      1
#define NOT_AVAILABLE   0

static int current = 0;
static int matrix_info = NOT_AVAILABLE;
static Viewer *viewer;

void child_wakeup_handler();
void child_data_handler();
void child_image_handler();
#ifdef USLEEP
int usleep(long usec);
#endif

void init_child_message_handler(Viewer *viewerinfo)
{
   viewer = viewerinfo;
   signal(SIGIO, child_data_handler);
   signal(SIGALRM, child_wakeup_handler);
   signal(SIGUSR2, child_image_handler);
}


void child_wakeup_handler()
{
   printf("Child is waking up!\n");
}

void child_data_handler()
{
   matrix_info = AVAILABLE;
}

void child_image_handler()
{
#if 0
   printf("child_image_handler()\n");
#endif
}

void child_main(Viewer *viewer)
{
   int            alarm_block_mask;
   int            alarm_unblock_mask;
   int            usr2_block_mask;
   int            usr2_unblock_mask;
   int            io_block_mask;
   int            io_unblock_mask;
   int            block_mask;
   int            unblock_mask;
   char           Command;
   LineData      *linedata = viewer->linedata;
   int            count = 0;

   int            i;
   int            j;
   int            width = viewer->width;
   int            height = viewer->height;
   int            depth = viewer->depth;
   int            Index;

   unsigned long *image24 = (unsigned long *) viewer->image->data;
   unsigned char *image8  = (unsigned char *) viewer->image->data;
   unsigned long  pixel;

   alarm_block_mask = sigmask(SIGALRM);
   usr2_block_mask = sigmask(SIGUSR2);
   io_block_mask = sigmask(SIGIO);

   block_mask = alarm_block_mask | usr2_block_mask | io_block_mask;
   unblock_mask = sigblock(block_mask);

   alarm_unblock_mask = usr2_block_mask | io_block_mask;
   usr2_unblock_mask = alarm_block_mask | io_block_mask;
   io_unblock_mask = alarm_block_mask | usr2_block_mask;

   /* Socket init */
#if 0
   printf("child() -- before sigpause\n");
#endif
   sigpause(alarm_unblock_mask);
#if 0
   printf("child() -- after sigpause\n");
#endif
   while (1)
   {
      kill(viewer->ppid, SIGUSR1);
      sigpause(io_unblock_mask);
      if (matrix_info == AVAILABLE)
      {
         recv_command_from_parent(&(viewer->comm), &Command);
#if 0
         printf("Command form parent = %c\n", Command);
#endif
         matrix_info = NOT_AVAILABLE;
      }

      usleep(250000);

      /* Update image */
#if 0
      printf("makeing image\n");
#endif
      if (count++ % 2)
      {
         if (depth == 8)
         {
            for (i = 0, Index = 0; i < height; i++)
               for (j = 0; j < width; j++, Index++)
                  image8[Index] = (i * j) % 256;
         }
         else
         {
            for (i = 0, Index = 0; i < height; i++)
               for (j = 0; j < width; j++, Index++)
               {
                  pixel = ((i * j) % 256) << 16;
                  image24[Index] = pixel;
               }
         }
      }
      else
      {
         if (depth == 8)
         {
            for (i = 0, Index = 0; i < height; i++)
               for (j = 0; j < width; j++, Index++)
                  image8[Index] = j % 256;
         }
         else
         {
            for (i = 0, Index = 0; i < height; i++)
               for (j = 0; j < width; j++, Index++)
               {
                  pixel = (i * j % 256) << 16;
                  image24[Index] = pixel;
               }
         }
      }

#if 0
      printf("done makeing image\n");
#endif
      kill(viewer->ppid, SIGUSR2);
      sigpause(usr2_unblock_mask);
   }
}

#ifdef USLEEP
int usleep(long usec)
{
   static struct
   {
      long tv_sec;
      long tv_usec;
   } delay;

   delay.tv_sec = usec / 1000000L;
   delay.tv_usec = usec % 1000000L;

   return(select(0, (long *) 0, (long *) 0, (long *) 0, &delay));
}
#endif
